{% from "schema_templates/schema_macros.jinja2" import badge %}

{% macro restriction(inner_text, css_class_name, html_id) %}
    <p>{{ badge('secondary', inner_text|safe, 'p-1') }}</p>
{% endmacro %}